#!/usr/bin/python
import ldap
import os
import pickle
import syslog
import sys

def ldapsync(uri, suffix, newlistpw, base,
oulists="ou=lists",
oupeople="ou=people",
userslist=None,
cacertfile=None,
uidaliases="/etc/postfix/uid-aliases",
mailaliases="/etc/mailman/aliases",
path2lists="/var/lib/mailman/lists",
genaliases="/usr/lib/mailman/bin/genaliases"):
	
	def dumplist(listname, members):
		"""helper function. Creates list if it does not exist and in any case dumps the associative array members in the members file of the list"""
		if not os.path.isdir(path2lists+"/"+listname):
			syslog.syslog("creating mailing list " + listname)
			os.system("echo | newlist {0} postmaster@{1} {2}".format(listname, suffix, newlistpw))
			os.system("sed s/LISTNAME/{0}/ {1}/extend.py > {1}/{0}/extend.py".format(listname, path2lists))
		with open("{0}/{1}/members".format(path2lists,listname), "w") as f:
			pickle.dump(members,f)
	
	syslog.openlog("ldap_sync")
	syslog.syslog("Syncing mailing lists")
	
	if cacertfile:
		ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)
		ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, cacertfile)
	
	people = []
	lists = []
	try:
		conn = ldap.initialize(uri)
		conn.simple_bind_s('','')
		lists = conn.search_s(oulists + "," + base, ldap.SCOPE_SUBTREE, "(objectClass=groupOfNames)", [])
		people = conn.search_s(oupeople + "," + base, ldap.SCOPE_SUBTREE, "(objectClass=*)", [])
		conn.unbind_s()
	except ldap.LDAPError as errors:
		for error in errors.args:
			syslog.syslog(syslog.LOG_ERR, str(error))
		return False
	
	
	# write the aliases file
	members={} #members of fs-users
	with open(uidaliases, "w") as f:
		f.write("#list automatically created by cronjob\n")
		for (dn,attrs) in people:
			if attrs.has_key("uid") and attrs.has_key("mail"):
				for uid in attrs["uid"]:
					f.write(uid + ": " + attrs["mail"][0] + "\n")
				#now create entries for global userslist
				mail = attrs["uid"][0] + "@" + suffix
				members[mail.lower()] = [ attrs["cn"] if attrs.has_key("cn") else None, mail ]
				
	
		
	# convert people to associative array
	# the keys are the DNs, so they can be found by the "member"-entries of the lists
	# the values are arrays of [name, E-Mail]
	recipients = {}
	for (dn, attrs) in people:
		if not attrs.has_key("mail"): continue
		recipients[dn] = [None, None]
		if attrs.has_key("cn"): recipients[dn][0] = attrs["cn"]
		if attrs.has_key("uid"):
			recipients[dn][1] = attrs["uid"][0] + "@" + suffix
		else:
			recipients[dn][1] = attrs["mail"][0]
	
	os.chdir(path2lists)
	
	#first create list for all users that have a uid
	if userslist: dumplist(userslist, members)
	#now for all other lists
	listnames = set([userslist] if userslist else [])
	for (dn, attrs) in lists:
		if not (attrs.has_key("cn") and "groupOfNames" in attrs["objectClass"]): continue
		listname = attrs["cn"][0]
		listnames.add(listname)
		members = {}
		for dn in attrs["member"]:
			if not recipients.has_key(dn): continue
			members[recipients[dn][1].lower()] = recipients[dn]
		dumplist(listname, members)
	
	#delete lists that are no longer in LDAP (except for USERSLIST)
	for filename in os.listdir(path2lists):
		if os.path.isdir(path2lists+"/"+filename) and not filename in listnames:
			os.system("rm -r {0}/{1}".format(path2lists,filename))
	
	os.system(genaliases + " | sed -n '5,$p' > " + mailaliases) 
	os.system("newaliases")
	return True
